"""
    Name: exception.py
    Author: Charles Zhang <694556046@qq.com>
    Propose: PyGrading exceptions
    Coding: UTF-8
"""


class DataTypeError(Exception):
    pass


class FunctionsTypeError(Exception):
    pass


class FieldMissingError(Exception):
    pass


class ExecError(Exception):
    pass


class FunctionArgsError(Exception):
    pass
